require 'tk'

#
# #########################################################
#
# #########################################################
#
# This is a simple code line counter. The user selects the path where the ruby source code files
# are stored usng the 'Path' menu button located in the top region of the window. The selected path is 
# stored to a global variable and displayed on the screen above a list of Ruby source code files stored
# at that location. The user can scroll up and down the list to select one or more files on which to
# perform the counting operation. A TK pushbutton immdeiately below the list allows the user to select
# either single or multiple files from the list.
#
# Two TK pushbuttons located just above the output area allow the user to select either all the files in
# the file list, or a subset of files for code line counting. When either pushbutton receives a mouse click,
# the first selected file from the list is opened and the code is counted as described below. Once the file
# has been completely read, the file name and the number of lines of Ruby code in that file are written to
# the output region using commas [between file name and code count] to allow spreadsheet importation.
# A running total of the counted code lines is displayed to the right of the file list. The next file in the
# list is processed in the same manner, and this continues until every selected file in the list has been
# counted. The total number of files counted is displayed to the right of the file list above the total lines
# of code counted.
#
# #########################################################
#
# The counting operation: 
#
# Spaces [blank lines] and Comment lines  can be either included in or excluded from the counting process.
# This is accomplished by clicking on the TK buttons located to the right of the file list.  The user clicks
# on them and the titles on the buttons change to indicate the counting/not counting  status.
#
# Spaces are what Ruby calls 'whitespace' characters like tabs, newlines, and stuff like that, basically
# blank. Lines that only contain whitespace characters are considered to be 'Spaces'. Lines that begin
# with optional contiguous whitespace characters followed by the comment character ['#'] will be
# identified as comment lines. Lines that have the first comment character ['#'] located 'inside' the
# code line [no optional leading whitespace characters] will not be considered a comment line. Regular
# expressions are used to identify Space and Comment lines.
#
# Each selected Ruby source code file is read line by line and counting can start on the next file read
# after the identification of the global EOT variable $\="\x03\n" which is present in every Ruby code
# source file.
#
# #########################################################
#
# #########################################################
#

#
# #########################################################
#
# Window drawing section: frames, labels, buttons, etc....
#
# #########################################################
#

font_name = "arial 10"

root = TkRoot.new() { title "CODE COUNTER" }
root.geometry('+300+300')

top_frame = TkFrame.new(root, :borderwidth=>5, :relief=>:ridge).pack(:side=>"top", "fill"=>"both", "expand"=>true, :pady=>5, :padx=>5)

path_label = TkLabel.new(top_frame, :relief=>:sunken, :background=>"lightgrey", :text=>"Source Code Path", "font"=>font_name).pack(:fill=>:x)

path_value_label = TkLabel.new(top_frame, :relief=>:sunken, :background=>"white", :text=>"", "font"=>font_name).pack(:fill=>:x)

middle_left_frame = TkFrame.new(top_frame, :borderwidth=>3, :relief=>:ridge).pack(:side=>"left", :pady=>5, :padx=>12)
middle_right_frame = TkFrame.new(top_frame, :borderwidth=>3, :relief=>:ridge).pack(:side=>"left", :pady=>5)

file_list_frame = TkFrame.new(middle_left_frame, :borderwidth=>3, :relief=>:ridge).pack(:side=>"top", :padx=>5)

file_list_label = TkLabel.new(file_list_frame, :relief=>:sunken, :background=>"lightgrey", :text=>"File List", "font"=>font_name).pack(:fill=>:x)

file_list_bar_vertical = TkScrollbar.new(file_list_frame).pack(:side=>"right", :fill=>"y")

file_list = TkListbox.new(file_list_frame, "background"=>"white", "exportselection" => 0, "width"=>20, "height"=>15, :selectmode=>"extended", "font"=>font_name).pack("side"=>"left")

file_list.yscrollbar(file_list_bar_vertical)

file_sel_button = TkButton.new(middle_left_frame, :background=>"lightgrey", :foreground=>"blue", :text=> " Select Single ", "font"=>font_name, "font"=>font_name, :width=>13).pack(:pady=>5)

total_files_frame = TkFrame.new(middle_right_frame).pack(:side=>:top, :fill=>:x)

total_files_label = TkLabel.new(total_files_frame, :text=>"Total Files Processed", :background=>"lightgrey", "font"=>font_name, :relief=>:sunken, :width=>16, :anchor=>:w).pack(:side=>:left, :fill=>:x)

total_files_count_label = TkLabel.new(total_files_frame, :text=>"", :background=>"white", :relief=>:sunken, :width=>9, :anchor=>:e).pack(:side=>:left, :fill=>:x)

total_code_frame = TkFrame.new(middle_right_frame).pack(:side=>:top, :fill=>:x, :pady=>10)

total_code_label = TkLabel.new(total_code_frame, :text=>"Total Lines of Code", :background=>"lightgrey", "font"=>font_name, :relief=>:sunken, :width=>16, :anchor=>:w).pack(:side=>:left, :fill=>:x)

total_code_count_label = TkLabel.new(total_code_frame, :text=>"", :background=>"white", :relief=>:sunken, :width=>9, :anchor=>:e).pack(:side=>:left, :fill=>:x)

spaces_toggle_button = TkButton.new(middle_right_frame, :background=>"lightgrey", :foreground=>"blue", :text=> "Not Counting Spaces", "font"=>font_name, "font"=>font_name, :width=>20).pack(:padx=>5)

comments_toggle_button = TkButton.new(middle_right_frame, :background=>"lightgrey", :foreground=>"blue", :text=> "Counting Comments", "font"=>font_name, :width=>20).pack(:pady=>10, :padx=>5)

middle_frame = TkFrame.new(root, :borderwidth=>5, :relief=>:ridge).pack("side"=>"top", "fill"=>"both", "expand"=>true, :padx=>5)

count_all_button = TkButton.new(middle_frame, :background=>"lightgrey", :foreground=>"blue", :text=> "Count All", "font"=>font_name, :width=>18).pack("side"=>"left", :padx=>10, :pady=>5)

count_selected_button = TkButton.new(middle_frame, :background=>"lightgrey", :foreground=>"blue", :text=> "Count Selected", "font"=>font_name, :width=>18).pack("side"=>"left", :pady=>5)

bottom_frame = TkFrame.new(root, :borderwidth=>5, :relief=>:ridge).pack("side"=>"top", "fill"=>"both", "expand"=>true, :pady =>5, :padx=>5)

output_label = TkLabel.new(bottom_frame, :relief=>:sunken, :background=>"lightgrey", :text=>"Code Lines Count Per Selected File - NOTE: ' Ctrl-/ ' selects all text.", "font"=>font_name).pack("fill"=>"x")

out_text_frame = TkFrame.new(bottom_frame).pack("side"=>"left", "fill"=>"both", "expand"=>true)

out_text_bar_vert = TkScrollbar.new(out_text_frame).pack("side"=>"right", "fill"=>"y")

out_text = TkText.new(out_text_frame,"width"=>60, "height"=>10, "font"=>font_name).pack("side"=>"bottom", "fill"=>"both", "expand"=>true)

out_text.yscrollbar(out_text_bar_vert)

#
# ###############################################################
#
# Globals:
#
# ###############################################################
#
# A default path starting point.
#
path = "C:/Users/"

#
# Global count-filter variables set via TK buttons.
#
no_spaces = (spaces_toggle_button.cget("text").eql?("Not Counting Spaces"))
no_comments = (comments_toggle_button.cget("text").eql?("Not Counting Comments"))

#
# ###############################################################
#
# Methods:
#
# ###############################################################
#

#
# For setting the selection mode for the file list.
#
def file_list_select_mode(file_list, sel_button)

	#
	# Determine the current selectmode configuration, it will be either "extended", only one item
	# selectable at a time, or "multiple", where more than one item at a time can be selected.
	# Switch configuration between the two: "extended" or "multiple".
	#
	case file_list.cget("selectmode")

		when "extended"

			file_list.configure("selectmode", "multiple")
			sel_button.configure("text", "Select Multiple")

		when "multiple"

			#
			# User can select (highlight) multiple list elements at once.
			#
			file_list.configure("selectmode", "extended")
			sel_button.configure("text", "Select Single")

			#
			# Clear all selected elements, then select (highlight) the last element selected,
			# which is the currently 'active' element - the last one clicked on.
			#
			file_list.selection_clear(0, "end")
			file_list.selection_set(file_list.index("active"))
	end
end

#
# This method counts the Ruby code source code lines in the file from the parameter list.
#
# The counting process commences when the EOT global var $\="\x03\n" has been read.
#
def count_ruby_code_lines(dir, file, out_text, no_spaces, no_comments)

	end_cnt = -1
	start_cnt = false
	code_cnt = 0

	File.open("#{dir}/#{file}", "r") do |src_file|

		while (line = src_file.gets)

			case (line)

				when (/\$\\="\\x03\\n"/)

					#
					# Our Ruby Code follows this.
					#
					start_cnt = true
				else
                         if (start_cnt)

                              ok_inc = true

                              ok_inc = false if (no_spaces and (line =~ /^\s+$/))
                              ok_inc = false if (no_comments and (line =~ /^\s*#/))
                              code_cnt += 1 if (ok_inc)
                         end
			end
		end
	end
 
     return code_cnt
end

def get_file_name(path)

	ftypes = [
		["Ruby source-files", '.rb'],
		["Ruby source files", '.rbw']
		]

	Tk.getOpenFile(:filetypes=>ftypes, :initialdir=>path)
end

#
# ###########################################################
#
# Menu:
#
#		Path:
#			Select a Path to the Ruby source code files.
#		Exit:
#			Exit the application.
#
# ###########################################################
#
main_menu = TkMenu.new(root)

main_menu.add('command', :label=> "Path", :command=> proc {

	
	fname = get_file_name(path)

	unless (fname.empty?)

		ext = File.extname(fname)
		path = File.dirname(fname)

		path_value_label.configure("text", "#{path}")
		total_files_count_label.configure("text", "")
		total_code_count_label.configure("text", "")
	
		file_list.delete(0, "end")

		#
		# Build a list of all Ruby source code files located at the chosen path.
		#
		Dir.foreach(path) { |fname|

			next if (!File.extname(fname).eql?(ext))
			file_list.insert("end", fname)
		}
	
		#
		# Select (show highlighting) first list element.
		#
		file_list.selection_set(0)
	end
})

main_menu.add('command', :label=> "Exit", :command=> proc{exit})

#
# Adding main_menu to the root window.
#
root.menu(main_menu)

#
# ###########################################################
#
# Button bindings:
#
# ###########################################################
#

#
# Set the selection mode for file_list:
#
file_sel_button.bind("Button-1") {

	file_list_select_mode(file_list, file_sel_button)
}

spaces_toggle_button.bind("Button-1") {

	#
	# User can toggle this between "Not Counting Spaces" and "Counting Spaces".
	#
	case spaces_toggle_button.cget("text")

		when ("Not Counting Spaces")
               spaces_toggle_button.configure("text", "Counting Spaces")
               no_spaces = false

		when ("Counting Spaces")
               spaces_toggle_button.configure("text", "Not Counting Spaces")
               no_spaces = true
	end
}

comments_toggle_button.bind("Button-1") {

	#
	# User can toggle this between "Not Counting Comments" and "Counting Comments".
	#
	case comments_toggle_button.cget("text")

		when ("Not Counting Comments")
               comments_toggle_button.configure("text", "Counting Comments")
               no_comments = false

		when ("Counting Comments")
               comments_toggle_button.configure("text", "Not Counting Comments")
               no_comments = true
	end
}

#
# This method counts the source code lines in all of the files displayed in the source code file list.
#
count_all_button.bind("Button-1") {

     file_name_array = Array.new

     file_name_array = file_list.get(0, "end")

     if (file_name_array.length > 0)

          out_text.delete("1.0", "end")
          total_code_cnt = code_cnt = 0

          file_name_array.each do |file|

               code_cnt = count_ruby_code_lines(path, file, out_text, no_spaces, no_comments)
               out_text.insert("end", "#{file},#{code_cnt}\n")
               total_code_cnt += code_cnt
          end

          total_files_count_label.configure("text", "#{file_name_array.length}")
          total_code_count_label.configure("text", "#{total_code_cnt}")

          out_text.focus
          out_text.mark_set("insert", "1.0")
     end
}

#
# This method counts the source code lines in the files currently selected from the file list.
#
count_selected_button.bind("Button-1") {

     index_array = Array.new

     index_array = file_list.curselection

     if (index_array.length > 0)

          out_text.delete("1.0", "end")
          total_code_cnt = code_cnt = 0

          index_array.each do |index|

               code_cnt = count_ruby_code_lines(path, file_list.get(index), out_text, no_spaces, no_comments)
               out_text.insert("end", "#{file_list.get(index)},#{code_cnt}\n")
               total_code_cnt += code_cnt
          end

          total_files_count_label.configure("text", "#{index_array.length}")
          total_code_count_label.configure("text", "#{total_code_cnt}")

          out_text.focus
          out_text.mark_set("insert", "1.0")
     end
}

Tk.mainloop()
